@@ -1,12 +1,13 @@
 import asyncio
 from typing import Dict, Optional, Union
+from warnings import warn
 
 from discord import Emoji, PartialEmoji
 from discord.abc import GuildChannel
 from discord.ext.commands import Context
 
 from dpymenus import BaseMenu
-from dpymenus.exceptions import ButtonsError
+from dpymenus.exceptions import ButtonsError, CallbackError
 
 
 class ButtonMenu(BaseMenu):
@@ -16,25 +17,26 @@ class ButtonMenu(BaseMenu):
     It contains Page objects which represent new Menu states that call methods for validation and handling.
 
     Attributes:
-        ctx: A reference to the command Context.
+        ctx: A reference to the command context.
         timeout: How long (in seconds) to wait before timing out.
-        state_fields: A dictionary containing dynamic state state_fields you wish to pass around the menu.
+        data: A dictionary containing variables to pass around menu functions.
     """
 
-    def __init__(self, ctx: Context, timeout: int = 300, state_fields: Optional[Dict] = None):
+    def __init__(self, ctx: Context, timeout: int = 300, data: Optional[Dict] = None):
         super().__init__(ctx, timeout)
-        self.state_fields = state_fields if state_fields else {}
+        self.data = data if data else {}
 
     def __repr__(self):
         return f'<Menu pages={[p.__str__() for p in self.pages]}, timeout={self.timeout}, ' \
-               f'active={self.active} page={self.page}, state_fields={self.state_fields}>'
+               f'active={self.active} page={self.page_index}, data={self.data}>'
 
     async def open(self):
         """The entry point to a new Menu instance. This will start a loop until a Page object with None as its function is set.
         Manages gathering user input, basic validation, sending messages, and cancellation requests."""
-        await super().open()
+        await super()._validate_pages()
+        await self._validate_buttons()
 
-        self.output = await self.ctx.send(embed=self.pages[self.page])
+        self.output = await self.ctx.send(embed=self.page)
 
         while self.active:
             await self._add_buttons()
@@ -42,17 +44,12 @@ class ButtonMenu(BaseMenu):
             self.input = await self._get_reaction()
             await self._cleanup_reactions()
 
-            await self.pages[self.page].callback(self)
-
-    async def close(self):
-        """Closes the active Menu instance."""
-        await self._cleanup_reactions()
-        self.active = False
+            await self.page.on_next(self)
 
     # Internal Methods
-    async def _add_buttons(self, ) -> None:
-        """Adds reactions to the message object based on what was passed into the Page's buttons."""
-        for button in self.pages[self.page].buttons:
+    async def _add_buttons(self):
+        """Adds reactions to the message object based on what was passed into the page buttons."""
+        for button in self.page.buttons:
             await self.output.add_reaction(button)
 
     async def _get_reaction(self) -> Union[Emoji, str]:
@@ -67,16 +64,30 @@ class ButtonMenu(BaseMenu):
                 return reaction.emoji.name
             return reaction.emoji
 
-    async def _cleanup_reactions(self) -> None:
+    async def _cleanup_reactions(self):
         """Removes all reactions from the output message object."""
         if isinstance(self.ctx.channel, GuildChannel):
             await self.output.clear_reactions()
 
-    def _validate_buttons(self):
+    async def _validate_buttons(self):
         """Ensures that a button menu was passed the appropriate amount of buttons."""
+        _cb_count = 0
         for page in self.pages:
-            if page.callback is None:
-                return
+            if page.buttons is None:
+                break
+
+            if page.on_next:
+                _cb_count += 1
 
             if len(page.buttons) <= 1:
-                raise ButtonsError('Your primary pages must have at least one button.')
+                raise ButtonsError('Any page with an `on_next` callback must have at least one button.')
+
+            if len(page.buttons) > 5:
+                warn('Adding more than 5 buttons to a page at once may result in discord.py throttling the bot client.')
+
+        if self.page.on_fail:
+            raise CallbackError('A ButtonMenu can not have an `on_fail` callback.')
+
+        if _cb_count < len(self.pages) - 1:
+            raise CallbackError(f'ButtonMenu missing `on_next` callbacks. Expected {len(self.pages) - 1}, found {_cb_count}.')
+
